@@ -13,15 +13,22 @@
 """
     librairies de fonctions communes
 """
-from os import system, makedirs, unlink
-from os.path import isdir, join, basename
+import codecs
+from os import system, makedirs, unlink, close, pardir, environ
+from os.path import isdir, join, basename, getsize, abspath
 from tempfile import mkstemp, mktemp
 from datetime import datetime, timedelta
 from shutil import move, rmtree
-from scribe import ldapconf
+from random import SystemRandom
+
+from scribe.ldapconf import MIN_PASSWORD_LENGTH, MIN_PASSWORD_CLASS, \
+     HOME_PATH, AD_HOME_PATH, EOLE_AD, RECYCLAGE_PATH, MAILDIR_PATH, CONTAINER_NAME
 from pyeole.process import system_code, system_out
-from pyeole.service import service_code
+from pyeole.service import unmanaged_service
+from pyeole.service.error import ServiceError
 from pyeole.deprecation import deprecated
+from fichier.passwd import check_nb_min_classes
+from fichier.acl import set_owner
 
 try:
     from creole.eosfunc import load_container_var
@@ -29,6 +36,8 @@ try:
 except:
     conteneurs = {}
 
+random = SystemRandom()
+
 def add_dir(dirname):
     """
     wrapper pour mkdir -p
@@ -48,10 +57,26 @@ def set_acl(dirname, groupname, mode='rwx', recursive=True):
     # l'option -P de setfacl permet d'ignorer les liens symboliques
     if recursive:
         cmd = ['setfacl', '-PRm', 'g:%s:%s' % (groupname, mode), '%s' % dirname]
-        system_code(cmd, container='fichier')
     else:
         cmd = ['setfacl', '-Pm', 'g:%s:%s' % (groupname, mode), '%s' % dirname]
-        system_code(cmd, container='fichier')
+    ret = system_out(cmd)
+    if ret[0] != 0:
+        print('error in set_acl: {} - {}'.format(' '.join(cmd), join(' - '.join(ret[1:]))))
+        print('return code: {}'.format(ret[0]))
+        if 'VM_DIR_EOLE_CI_TEST' in environ and ret[0] == 2:
+            # on est dans le serveur de test EOLE
+            # a priori il ne retrouve pas l'utilisateur
+            print()
+            print('* wbinfo -u')
+            system_code(['wbinfo', '-u'])
+            print()
+            print('* wbinfo -g')
+            system_code(['wbinfo', '-g'])
+            print()
+            print('* systemctl status ')
+            system_code(['systemctl', 'status', 'winbind.service', '--no-pager'])
+    elif ret[1]:
+        print(ret[1])
 
 @deprecated
 def set_default_acl(dirname, recursive=True):
@@ -76,23 +101,32 @@ def convert_file(filename):
     """
     conversion du fichier en UTF-8
     (si nécessaire)
+    retourne le type de format détecté (utf8 ou utf8 avec bom)
     """
+    begin = min(32, getsize(filename))
+    raw = open(filename, 'rb').read(begin)
+
+    if raw.startswith(codecs.BOM_UTF8):
+        return "utf-8-sig"
+
     cmd = '/usr/bin/iconv'
-    cont = file(filename, 'r').read()
+    cont = open(filename, 'rb').read()
     try:
-        unicode(cont, 'utf8')
+        cont.decode('utf8')
         # déjà en UTF-8
-        return True
+        return "utf-8"
     except:
-        try:
-            # fichier en ISO
-            unicode(cont, 'iso8859-1')
-            tmpf = mktemp()
-            system('%s -f iso8859-1 -t utf8 "%s" > "%s"' % (cmd, filename, tmpf))
-            system('/bin/mv -f "%s" "%s"' % (tmpf, filename))
-        except:
-            raise Exception("L'encodage du fichier %s n'est pas supporté !")
-    return True
+        pass
+    try:
+        # fichier en ISO
+        cont.decode('iso8859-1')
+        tmpf = mktemp()
+        system('%s -f iso8859-1 -t utf8 "%s" > "%s"' % (cmd, filename, tmpf))
+        system('/bin/mv -f "%s" "%s"' % (tmpf, filename))
+        return "utf-8"
+    except:
+        raise UnicodeError("Tentative de conversion de fichier impossible !")
+
 
 def ok_groupe(groupe, gtype='g'):
     """
@@ -127,8 +161,7 @@ def ok_groupe(groupe, gtype='g'):
     # FIXME
     if len(groupe) > 20:
         return groupe[0:20]
-    else:
-        return groupe
+    return groupe
 
 def replace_cars(chaine):
     """
@@ -136,8 +169,6 @@ def replace_cars(chaine):
     """
     if chaine is None:
         return ''
-    if type(chaine) == unicode:
-        chaine = chaine.encode('UTF-8')
     chars = {'¥' : 'Y', 'µ' : 'u', 'À' : 'A', 'Á' : 'A',
              'Â' : 'A', 'Ã' : 'A', 'Ä' : 'A', 'Å' : 'A',
              'Æ' : 'A', 'Ç' : 'C', 'È' : 'E', 'É' : 'E',
@@ -154,8 +185,10 @@ def replace_cars(chaine):
              'ô' : 'o', 'õ' : 'o', 'ö' : 'o', 'ø' : 'o',
              'ù' : 'u', 'ú' : 'u', 'û' : 'u', 'ü' : 'u',
              'ý' : 'y', 'ÿ' : 'y', '~B': 'e', '°' : '.',
+             'Ō' : 'O',
+             '\\': '|', 'Œ' : 'OE', 'œ' : 'oe',
             }
-    for old, new in chars.items():
+    for old, new in list(chars.items()):
         chaine = chaine.replace(old, new)
     return chaine
 
@@ -188,7 +221,8 @@ def manage_nom(nom):
     """ gestion des noms à particule """
     nom = nom.lower().replace("'", '')
     particules = ['de', 'du', 'dela', 'desa', 'dos', 'di', 'es', 'el', 'le',
-                  'la', 'da', 'van', 'von', 'ben', 'saint', 'ez', 'ait', 'sa']
+                  'la', 'da', 'van', 'von', 'ben', 'saint', 'ez', 'ait', 'sa',
+                  'del', 'et']
     for particule in particules:
         if nom.startswith('%s ' % particule):
             nom = nom.replace('%s ' % particule, particule, 1)
@@ -201,7 +235,7 @@ def manage_nom(nom):
 def manage_prenom(prenom):
     """ gestion des prénoms à particule """
     prenom = prenom.lower().replace("'", '')
-    particules = ['el']
+    particules = ['el', 'my']
     for particule in particules:
         if prenom.startswith('%s ' % particule):
             prenom = prenom.replace('%s ' % particule, particule, 1)
@@ -248,6 +282,44 @@ def gen_login(pref, prenom, nom):
         login = login[0:19]
     return login
 
+
+def valid_password(pwd):
+    """
+    Vérifie la validité d'un mot de passe
+    selon les critères définis dans la configuration du module
+    """
+    return len(pwd) >= MIN_PASSWORD_LENGTH and check_nb_min_classes(pwd, MIN_PASSWORD_CLASS)
+
+
+def gen_strong_password():
+    """
+    Génération d'un mot de passe aléatoire solide
+    (8 caractères + 3 classes minimum)
+    """
+    def random_letter(upper=False):
+        "[a-z] ou [A-Z]"
+        letter = random.choice('abcdefghijklmnopqrstuvwxyz')
+        return letter.upper() if upper else letter
+    def random_upper():
+        "[A-Z]"
+        return random_letter(upper=True)
+    def random_number():
+        "[0-9]"
+        return str(random.randint(0, 9))
+    def random_special_cars():
+        "our special chars"
+        return random.choice('&$@%=')
+    random_factory = {
+            1: random_letter,
+            2: random_upper,
+            3: random_number,
+            4: random_special_cars}
+    # let's decide how to populate the passwd string
+    # default weighting ([a-z] : 4, [A-Z] : 3, [0-9] : 2, other : 2)
+    weighting = [1, 1, 1, 1, 2, 2, 2, 3, 3, 4, 4]
+    return "".join([random_factory[n]() for n in random.sample(weighting, 8)])
+
+
 def format_current_date(format="%Y%m%d"):
     """
     renvoie la date du jour formatée
@@ -268,10 +340,9 @@ def is_empty(dico, cle):
     """
     pas de clé ou valeur vide
     """
-    if dico.has_key(cle) and dico[cle] != '':
+    if cle in dico and dico[cle] != '':
         return False
-    else:
-        return True
+    return True
 
 def not_empty(dico, cle):
     """
@@ -288,8 +359,7 @@ def formate_civilite(titre):
         return '3'
     elif titre.lower() == 'mme':
         return '2'
-    else:
-        return '1'
+    return '1'
 
 def to_list(data):
     """
@@ -299,10 +369,9 @@ def to_list(data):
         return data
     elif data == '':
         return []
-    elif not hasattr(data, '__iter__'):
+    elif not hasattr(data, '__iter__') or isinstance(data, str):
         return list(set([data]))
-    else:
-        return list(set(data))
+    return list(set(data))
 
 def formate_date(date, sep='/'):
     """
@@ -325,8 +394,7 @@ def formate_date(date, sep='/'):
     elif len(day) == 4:
         # la date était déjà à l'anglaise !
         return sep.join((year, month, day))
-    else:
-        return date
+    return date
 
 def clean_date(date):
     """
@@ -334,6 +402,36 @@ def clean_date(date):
     """
     return date.replace('/','').replace('-','').replace(' ', '')
 
+def get_days_from_epoch(date):
+    """
+    donne le nombre de jour entre epoch et une date au format
+    jj/mm/aaaa
+    """
+    target_date = datetime(year=int(date[6:10]), month=int(date[3:5]), day=int(date[0:2]))
+    return (target_date - datetime(1970, 1, 1)).days
+
+def get_date_from_epoch(unix_time):
+    """
+    donne la donne epoch + days au format jj/mm/aaaa
+    """
+    date = datetime.fromtimestamp(unix_time)
+    return date.strftime('%d/%m/%Y')
+
+def get_nt_time_from_date(date):
+    """
+    donne la durée en centaine de ns depuis une date
+    jj/mm/aaaa
+    """
+    target_date = datetime(year=int(date[6:10]), month=int(date[3:5]), day=int(date[0:2]))
+    return (target_date - datetime(1601, 1, 1)).days * 24 * 3600 * 10000000
+
+def get_date_from_nt_time(nt_time):
+    """
+    donne la date 1/1/1601 + centaines de ns au format jj/mm/aaaa
+    """
+    date = datetime(year=1601, month=1, day=1) + timedelta(seconds=nt_time/10000000)
+    return date.strftime('%d/%m/%Y')
+
 def deformate_date(date):
     """
     formate une date en aaaammjj pour l'annuaire
@@ -345,8 +443,7 @@ def deformate_date(date):
         return date[4:8]+date[2:4]+date[0:2]
     elif len(date) == 6:
         return date[4:6]+date[2:4]+date[0:2]
-    else:
-        raise Exception("format de date incorrect : %s" % date)
+    raise Exception("format de date incorrect : %s" % date)
 
 def fixdate(date):
     """
@@ -373,20 +470,57 @@ def strip_adresse(adresse):
     adresse = adresse.replace('\n', '').replace(' ', '').replace(',', '')
     return adresse.replace('.', '').lower()
 
+def actualise_cache():
+    """ démarrage de LSC """
+    if EOLE_AD:
+        cmd = ['/usr/bin/actualise_cache']
+        ret = system_out(cmd)
+        if ret[0] != 0:
+            print('error in actualise_cache: {} - {}'.format(' '.join(cmd), ' - '.join(ret[1:])))
+            print('return code: {}'.format(ret[0]))
+
 def nscd_start():
-    """ démarrage de nscd """
-    service_code('nscd', 'start', 'fichier')
+    """ démarrage de LSC """
+    if EOLE_AD:
+        actualise_cache()
+        unmanaged_service('start', 'eole-lsc', 'service')
 
 def nscd_stop():
-    """ arrêt de nscd """
-    service_code('nscd', 'stop', 'fichier')
+    """ arrêt de LSC """
+    if EOLE_AD:
+        unmanaged_service('stop', 'eole-lsc', 'service')
 
-def my_escape(string):
-    """
-    remplacement de caractères pour dyn-logon
-    -> ne pas remplacer les "."
-    """
-    return string.strip().replace(' ','').replace('"', "\'").replace(';', '\;').replace("'", "\\'")
+def lsc_sync():
+    if EOLE_AD:
+        cmd = ['lsc', '-f', '/etc/lsc', '-s', 'all', '-t1']
+        ret = system_out(cmd)
+        if ret[0] != 0:
+            print('error in lsc_sync: {} - {}'.format(' '.join(cmd), ' - '.join(ret[1:])))
+            print('return code: {}'.format(ret[0]))
+        else:
+            try:
+                unmanaged_service('status', 'eole-lsc', 'service')
+                nscd_stop()
+                nscd_start()
+            except ServiceError:
+                # ne pas redémarrer lsc
+                actualise_cache()
+
+def lsc_full_sync():
+    if EOLE_AD:
+        cmd = ['lsc', '-f', '/etc/lsc', '-s', 'all', '-c', 'all']
+        ret = system_out(cmd)
+        if ret[0] != 0:
+            print('error in lsc_full_sync: {} - {}'.format(' '.join(cmd), ' - '.join(ret[1:])))
+            print('return code: {}'.format(ret[0]))
+        else:
+            try:
+                unmanaged_service('status', 'eole-lsc', 'service')
+                nscd_stop()
+                nscd_start()
+            except ServiceError:
+                # ne pas redémarrer lsc
+                actualise_cache()
 
 def calc_recycle_dir(subdir):
     """
@@ -394,7 +528,7 @@ def calc_recycle_dir(subdir):
     /home/recyclage/<année>/<subdir>
     """
     current_year = format_current_date(format="%Y")
-    recycle_dir = join(ldapconf.RECYCLAGE_PATH, current_year, subdir)
+    recycle_dir = join(RECYCLAGE_PATH, current_year, subdir)
     add_dir(recycle_dir)
     return recycle_dir
 
@@ -410,6 +544,23 @@ def calc_dest_dir(base_dir):
         num += 1
     return dest_dir
 
+def create_ad_home(login, home):
+    """
+    Déplace le home et crée le lien
+    """
+    ad_home = join(AD_HOME_PATH, login)
+    abc = abspath(join(home, pardir))
+    add_dir(abc)
+    if not isdir(ad_home):
+        if isdir(home):
+            move(home, ad_home)
+        else:
+            makedirs(ad_home, 0o700)
+        set_owner(ad_home, login)
+        system('chmod u+w {}'.format(ad_home))
+        add_symlink(ad_home, home)
+    # else: #FIXME
+
 def move_user_datas(user):
     """
     déplacement des données d'un utilisateur supprimé
@@ -417,20 +568,28 @@ def move_user_datas(user):
     """
     recycle_dir = calc_recycle_dir(user[0])
     dest_dir = calc_dest_dir(join(recycle_dir, user))
-    old_user_dir = join(ldapconf.HOME_PATH, user[0], user)
-    if isdir(old_user_dir):
-        # d'abord un peu de ménage (#2101)
-        rmtree(join(old_user_dir, '.ftp'), ignore_errors=True)
-        rmtree(join(old_user_dir, 'groupes'), ignore_errors=True)
-        system('find "%s" -type l -delete' % old_user_dir)
-        move(old_user_dir, dest_dir)
+    old_user_dir = join(HOME_PATH, user[0], user)
+    ad_user_dir = join(AD_HOME_PATH, user)
+    if isdir(ad_user_dir):
+        real_user_dir = ad_user_dir
+        # remove symlink
+        unlink(old_user_dir)
+    elif isdir(old_user_dir):
+        real_user_dir = old_user_dir
+    else:
+        return
+    # d'abord un peu de ménage (#2101)
+    rmtree(join(real_user_dir, '.ftp'), ignore_errors=True)
+    rmtree(join(real_user_dir, 'groupes'), ignore_errors=True)
+    system('find "%s" -type l -delete' % real_user_dir)
+    move(real_user_dir, dest_dir)
 
 def move_mail_datas(user):
     """
     déplacement des données d'un compte Mail supprimé
     sans demande de suppression de ses données
     """
-    old_user_dir = join(ldapconf.MAILDIR_PATH, user)
+    old_user_dir = join(MAILDIR_PATH, user)
     if isdir(old_user_dir):
         recycle_dir = calc_recycle_dir('mail')
         dest_dir = calc_dest_dir(join(recycle_dir, user))
@@ -453,13 +612,13 @@ def launch_smbldap_tool(cmd, ref_etab, etab, force_dn={}):
     if etab != None and ref_etab != etab:
         filename = '%s/etc/smbldap-tools/smbldap.conf' % conteneurs.get('container_path_fichier', '')
         confdir = '/etc/smbldap-tools/'
-        tempfile = mkstemp(suffix='.temp', dir='%s%s' % (conteneurs.get('container_path_fichier', ''), confdir))[1]
+        fd, tempfile = mkstemp(suffix='.temp', dir='%s%s' % (conteneurs.get('container_path_fichier', ''), confdir))
         container_file = '%s%s' % (confdir, basename(tempfile))
         fh = open(filename, 'r')
         filecontent = fh.read()
         fh.close()
         newcontent = filecontent.replace('num_etab="{0}"'.format(ref_etab), 'num_etab="{0}"'.format(etab))
-        for old, new in force_dn.items():
+        for old, new in list(force_dn.items()):
             newcontent = newcontent.replace(old, new)
         fh = open(tempfile, 'w')
         fh.write(newcontent)
@@ -467,8 +626,17 @@ def launch_smbldap_tool(cmd, ref_etab, etab, force_dn={}):
         env = {'SMBLDAP_CONF': container_file}
     else:
         env = None
-    res = system_out(cmd, container='fichier', env=env)
+    res = system_out(cmd, container=CONTAINER_NAME, env=env)
     if tempfile is not None:
+        close(fd)
         unlink(tempfile)
     if res[0] != 0:
-        raise Exception, "Erreur à l'exécution de la commande %s : %s" % (' '.join(cmd), ''.join(res[1:]))
+        raise Exception("Erreur à l'exécution de la commande %s : %s" % (' '.join(cmd), ''.join(res[1:])))
+
+
+def get_reverse_login(login):
+    if '.' not in login:
+        return login
+    rev_login = login.split('.', 1)
+    rev_login.reverse()
+    return '.'.join(rev_login)
